#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vector>

#include "global.h"

/* TODO: fix those */
#include "camera.h"
#include "mesh.h"
#include "object.h"
#include "scene.h"
#include "shader_manager.h"

#include "opengl/opengl.h"
#include "vulkan/vk.h"

/* static functions */

static bool init();
static void cleanup();
static void display();

/* glfw callbacks */
static void key_clbk(GLFWwindow *win, int key, int scancode, int action, int mods);
static void motion_clbk(GLFWwindow *win, double x, double y);
static void mouse_clbk(GLFWwindow *win, int button, int action, int mods);

/* global variables */
bool use_vulkan;

GLFWwindow *win;
int win_w = 800;
int win_h = 600;

OrbitCamera *camera;
ShaderManager *sdr_man;

/* variables */
static float phi = 25;
static float theta = 0;
static float dist = 4;

// TODO: remove just for test:
static Scene scene;

int main(int argc, char **argv)
{
	for(int i=0; i<argc; ++i) {
		if(strcmp(argv[i], "-opengl") == 0) {
			use_vulkan = false;
			printf("Backend: OpenGL.\n");
		}
		else if(strcmp(argv[i], "-vulkan") == 0) {
			use_vulkan = true;
			printf("Backend: Vulkan.\n");
		}
		else {
			printf("No backend specified. Using OpenGL.\n");
		}
	}

	if(!init()) {
		fprintf(stderr, "Failed to initialize program.\n");
		return 1;
	}

	glfwSetKeyCallback(win, key_clbk);
	glfwSetCursorPosCallback(win, motion_clbk);
	glfwSetMouseButtonCallback(win, mouse_clbk);

	while(!glfwWindowShouldClose(win)) {
		display();

		glfwSwapBuffers(win);
		glfwPollEvents();
	}

	atexit(cleanup);
	return 0;
}

static bool init()
{
	/* TODO */
	/*
		TODO changes:
		1- create cam
		2- scene
		3- renderers
	*/
	if(use_vulkan) {
		if(!init_vulkan())
			return false;
	}
	else {
		if(!init_opengl())
			return false;
	}

	sdr_man = new ShaderManager;

	camera = new OrbitCamera;
	camera->set_orbit_params(phi, theta, dist);

	if(!scene.load("data/spot/spot_control_mesh.obj")) {
		fprintf(stderr, "Failed to load scene.\n");
		return false;
	}

	for(size_t i=0; i<scene.objects.size(); ++i) {
		printf("object: %d\n", (int)i);
		printf("mesh: %s\n", scene.objects[i]->mesh->name.c_str());
		printf("material: %s\n", scene.objects[i]->material->name.c_str());
		printf("transform:\n");
		scene.objects[i]->transform.print();
	}
	return true;
}

static void cleanup()
{
	delete sdr_man;
	delete camera;

	if(use_vulkan) {
		cleanup_vulkan();
	}
	else {
		cleanup_opengl();
	}
}

static void key_clbk(GLFWwindow *win, int key, int scancode, int action, int mods)
{
	if(key == GLFW_KEY_ESCAPE && action == GLFW_PRESS) {
		glfwSetWindowShouldClose(win, GLFW_TRUE);
	}
}

static double prev_x, prev_y;
static int button;

static void motion_clbk(GLFWwindow *win, double x, double y)
{
	switch(button) {
	case GLFW_MOUSE_BUTTON_LEFT:
		theta += x - prev_x;
		phi += y - prev_y;

		if(phi < -90)
			phi = -90;
		if(phi > 90)
			phi = 90;

		break;

	case GLFW_MOUSE_BUTTON_RIGHT:
		dist *= (y - prev_y) * 0.01 + 1;
		if(dist < 0.0) {
			dist = 0.0;
		}
		break;
	}

	prev_x = x;
	prev_y = y;
}

static void mouse_clbk(GLFWwindow *win, int bn, int action, int mods)
{
	button = bn;
	glfwGetCursorPos(win, &prev_x, &prev_y);
}

static void display()
{
	if(use_vulkan) {
		display_vulkan();
	}
	else {
		display_opengl();
		scene.objects[0]->mesh->draw();
	}
}